@@ -18,18 +18,18 @@ from typing import Union, Sequence
 
 import numpy as np
 
-from jax._src.api import jit, linear_transpose, ShapeDtypeStruct
-from jax.core import Primitive
 from jax.interpreters import mlir
 from jax.interpreters import xla
-from jax._src.util import prod
 from jax import lax
 from jax.interpreters import ad
 from jax.interpreters import batching
-from jax._src.lib.mlir.dialects import mhlo
+from jax._src.api import jit, linear_transpose, ShapeDtypeStruct
+from jax._src.core import Primitive, is_constant_shape
+from jax._src.lib.mlir.dialects import hlo
 from jax._src.lib import xla_client
 from jax._src.lib import ducc_fft
 from jax._src.numpy.util import _promote_dtypes_complex, _promote_dtypes_inexact
+from jax._src.util import prod
 
 __all__ = [
   "fft",
@@ -103,16 +103,21 @@ def fft_abstract_eval(x, fft_type, fft_lengths):
   return x.update(shape=shape, dtype=dtype)
 
 def _fft_lowering(ctx, x, *, fft_type, fft_lengths):
-  out_aval, = ctx.avals_out
   return [
-      mhlo.FftOp(x, mhlo.FftTypeAttr.get(fft_type.name),
-                  mlir.dense_int_elements(fft_lengths)).result
+      hlo.FftOp(x, hlo.FftTypeAttr.get(fft_type.name),
+                mlir.dense_int_elements(fft_lengths)).result
   ]
 
 
 def _fft_lowering_cpu(ctx, x, *, fft_type, fft_lengths):
+  if any(not is_constant_shape(a.shape) for a in (ctx.avals_in + ctx.avals_out)):
+    raise NotImplementedError("Shape polymorphism for custom call is not implemented (fft); b/261671778")
   x_aval, = ctx.avals_in
-  return [ducc_fft.ducc_fft_mhlo(x, x_aval.dtype, fft_type=fft_type,
+  if xla_client.mlir_api_version < 41:
+    return [ducc_fft.ducc_fft_mhlo(x, x_aval.dtype, fft_type=fft_type,
+                                   fft_lengths=fft_lengths)]
+  else:
+    return [ducc_fft.ducc_fft_hlo(x, x_aval.dtype, fft_type=fft_type,
                                   fft_lengths=fft_lengths)]
 
 def _naive_rfft(x, fft_lengths):
