@@ -21,5 +21,11 @@ import jaxlib.mlir.dialects.ml_program as ml_program
 import jaxlib.mlir.dialects.sparse_tensor as sparse_tensor
 
 from jax.lib import xla_client
-if xla_client.mlir_api_version >= 37:
-  import jaxlib.mlir.dialects.stablehlo as stablehlo
+import jaxlib.mlir.dialects.stablehlo as stablehlo
+
+# Alias that is set up to abstract away the transition from MHLO to StableHLO.
+use_stablehlo = xla_client.mlir_api_version >= 42
+if use_stablehlo:
+  import jaxlib.mlir.dialects.stablehlo as hlo
+else:
+  import jaxlib.mlir.dialects.mhlo as hlo  # type: ignore[no-redef]
