@@ -157,6 +157,10 @@ def _compute_fans(shape: core.NamedShape,
   Axes not in in_axis, out_axis, or batch_axis are assumed to constitute the
   "receptive field" of a convolution (kernel spatial dimensions).
   """
+  if shape.rank <= 1:
+    raise ValueError(f"Can't compute input and output sizes of a {shape.rank}"
+                     "-dimensional weights tensor. Must be at least 2D.")
+
   if isinstance(in_axis, int):
     in_size = shape[in_axis]
   else:
