@@ -19,8 +19,7 @@ import numpy as np
 import scipy.linalg
 import textwrap
 import warnings
-from typing import cast, overload, Any, Optional, Tuple, Union
-from typing_extensions import Literal
+from typing import cast, overload, Any, Literal, Optional, Tuple, Union
 
 import jax
 from jax import jit, vmap, jvp
@@ -28,6 +27,7 @@ from jax import lax
 from jax._src import dtypes
 from jax._src.lax import linalg as lax_linalg
 from jax._src.lax import qdwh
+from jax._src.numpy.lax_numpy import _check_arraylike
 from jax._src.numpy.util import _wraps, _promote_dtypes_inexact, _promote_dtypes_complex
 from jax._src.numpy import lax_numpy as jnp
 from jax._src.numpy import linalg as np_linalg
@@ -203,7 +203,7 @@ def _schur(a: Array, output: str) -> Tuple[Array, Array]:
 def schur(a: ArrayLike, output: str = 'real') -> Tuple[Array, Array]:
   if output not in ('real', 'complex'):
     raise ValueError(
-      f"Expected 'output' to be either 'real' or 'complex', got output={output}.")
+      f"Expected 'output' to be either 'real' or 'complex', got {output=}.")
   return _schur(a, output)
 
 @_wraps(scipy.linalg.inv,
@@ -858,7 +858,7 @@ def polar(a: ArrayLike, side: str = 'right', *, method: str = 'qdwh', eps: Optio
     else:
       raise NotImplementedError("method='qdwh' only supports mxn matrices "
                                 "where m < n where side='right' and m >= n "
-                                f"side='left', got {a.shape} with side={side}")
+                                f"side='left', got {a.shape} with {side=}")
   elif method == "svd":
     u_svd, s_svd, vh_svd = lax_linalg.svd(a, full_matrices=False)
     s_svd = s_svd.astype(u_svd.dtype)
@@ -1032,3 +1032,28 @@ def hessenberg(a: ArrayLike, *, calc_q: bool = False, overwrite_a: bool = False,
     return h, q
   else:
     return h
+
+@_wraps(scipy.linalg.toeplitz)
+def toeplitz(c: ArrayLike, r: Optional[ArrayLike] = None) -> Array:
+  if r is None:
+    _check_arraylike("toeplitz", c)
+    r = jnp.conjugate(jnp.asarray(c))
+  else:
+    _check_arraylike("toeplitz", c, r)
+
+  c = jnp.asarray(c).flatten()
+  r = jnp.asarray(r).flatten()
+
+  ncols, = c.shape
+  nrows, = r.shape
+
+  if ncols == 0 or nrows == 0:
+    return jnp.empty((ncols, nrows), dtype=jnp.promote_types(c.dtype, r.dtype))
+
+  nelems = ncols + nrows - 1
+  elems = jnp.concatenate((c[::-1], r[1:]))
+  patches = lax.conv_general_dilated_patches(
+      elems.reshape((1, nelems, 1)),
+      (nrows,), (1,), 'VALID', dimension_numbers=('NTC', 'IOT', 'NTC'),
+      precision=lax.Precision.HIGHEST)[0]
+  return jnp.flip(patches, axis=0)
