@@ -12,19 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Any, Callable, Tuple
-
 import jax
 from jax import core
 from jax import tree_util
-from jax import linear_util as lu
+from jax._src import linear_util as lu
 from jax.experimental import pjit
 
-from jax._src.lib.mlir.dialects import mhlo
+from jax._src.lib.mlir.dialects import hlo
 from jax._src.lib.mlir import ir
 import jax.interpreters.pxla as pxla
 from jax.interpreters import mlir
-from jax.interpreters import xla
 from jax.interpreters import partial_eval as pe
 from jax._src import custom_api_util
 from jax._src.lib import xla_client as xc
@@ -245,7 +242,7 @@ def _custom_partitioning_lowering_rule(ctx: mlir.LoweringRuleContext, *values,
   else:
     out_type = [ir.TupleType.get_tuple(mlir_shapes)]
 
-  out = mhlo.CustomCallOp(
+  out = hlo.CustomCallOp(
       out_type,
       list(values),
       call_target_name=ir.StringAttr.get(_CUSTOM_PARTITIONING_CALL_NAME),
@@ -259,7 +256,7 @@ def _custom_partitioning_lowering_rule(ctx: mlir.LoweringRuleContext, *values,
     return [out.result]
   else:
     return [
-        mhlo.GetTupleElementOp(out, mlir.i32_attr(i)).result
+        hlo.GetTupleElementOp(out, mlir.i32_attr(i)).result
         for i in range(len(mlir_shapes))
     ]
 
