@@ -109,21 +109,23 @@ def _spec_has_metadata(tree):
   return 'metadata' in tree or any(
       _spec_has_metadata(subtree) for _, subtree in tree.items())
 
+def _get_kvstore_for_gcs(ckpt_path: str):
+  m = re.fullmatch('^gs://([^/]*)/(.*)$', ckpt_path, re.DOTALL)
+  if m is None:
+    raise ValueError('The ckpt_path should contain the bucket name and the '
+                      f'file path inside the bucket. Got: {ckpt_path}')
+  gcs_bucket = m.group(1)
+  path_without_bucket = m.group(2)
+  return {'driver': 'gcs', 'bucket': gcs_bucket, 'path': path_without_bucket}
 
 def get_tensorstore_spec(ckpt_path: str):
   spec = {'driver': 'zarr', 'kvstore': {}}
 
   if ckpt_path.startswith('gs://'):
-    m = re.fullmatch('^gs://([^/]*)/(.*)$', ckpt_path, re.DOTALL)
-    if m is None:
-      raise ValueError('The ckpt_path should contain the bucket name and the '
-                       f'file path inside the bucket. Got: {ckpt_path}')
-    gcs_bucket = m.group(1)
-    path_without_bucket = m.group(2)
-    spec['kvstore'] = {'driver': 'gcs', 'bucket': gcs_bucket,
-                       'path': path_without_bucket}
+    spec['kvstore'] = _get_kvstore_for_gcs(ckpt_path)
   else:
     spec['kvstore'] = {'driver': 'file', 'path': ckpt_path}
+
   return spec
 
 
