@@ -14,26 +14,28 @@
 
 import contextlib
 import functools
-from functools import partial
 import itertools as it
+from functools import partial
 from typing import Any, Callable, Dict, List, Tuple, Sequence, Optional, Union
+
 import jax
+from jax._src import linear_util as lu
 from jax.interpreters import partial_eval as pe
 from jax.config import config
-from jax import core
-from jax._src.dtypes import dtype, float0
-from jax.core import (Trace, Tracer, get_aval, call_p, Primitive, Literal,
-                      raise_to_shaped)
+from jax.tree_util import (tree_flatten, tree_unflatten,
+                           register_pytree_node, Partial)
+from jax._src import core
+from jax._src import source_info_util
 from jax._src.ad_util import (add_jaxvals, add_jaxvals_p, zeros_like_jaxval,
                               zeros_like_aval, zeros_like_p, Zero)
+from jax._src.api_util import flatten_fun, flatten_fun_nokwargs
+from jax._src.core import (Trace, Tracer, get_aval, call_p, Primitive, Literal,
+                           raise_to_shaped)
+from jax._src.dtypes import dtype, float0
 from jax._src.util import (unzip2, safe_map, safe_zip, split_list, wrap_name,
                            as_hashable_function, weakref_lru_cache,
                            partition_list)
-from jax.tree_util import register_pytree_node
-from jax import linear_util as lu
-from jax._src.api_util import flatten_fun, flatten_fun_nokwargs
-from jax.tree_util import tree_flatten, tree_unflatten, Partial
-from jax._src import source_info_util
+
 
 zip = safe_zip
 map = safe_map
