@@ -12,8 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Lowering and execution path that converts jaxprs into the MLIR MHLO/CHLO
-# dialects.
+# Lowering and execution path that converts jaxprs into MLIR.
 from __future__ import annotations
 
 import collections
@@ -22,32 +21,33 @@ import functools
 from functools import partial
 import io
 import itertools
+import operator
 import re
 import typing
 from typing import (Any, Callable, Dict, Iterator, List, NamedTuple, Optional,
                     Protocol, Sequence, Set, Tuple, Type, Union, FrozenSet)
 import warnings
 
-from jax import core
-from jax import linear_util as lu
+import numpy as np
+
+from jax._src import linear_util as lu
+from jax.config import config
+from jax.interpreters import ad
+from jax.interpreters import partial_eval as pe
+from jax.interpreters import xla
 from jax._src import ad_util
+from jax._src import core
 from jax._src import device_array
 from jax._src import dtypes
-from jax._src.lib import mlir_api_version, xla_extension_version
-from jax._src.lib.mlir import ir
-from jax._src.lib.mlir.dialects import chlo
-from jax._src.lib.mlir.dialects import mhlo
-from jax._src.lib.mlir.dialects import func as func_dialect
-from jax._src.lib import can_execute_with_token
+from jax._src import source_info_util
+from jax._src import util
+from jax._src.lib import mlir_api_version
 from jax._src.lib import xla_bridge as xb
 from jax._src.lib import xla_client as xc
-from jax._src import source_info_util
-import jax._src.util as util
-from jax.config import config
-import jax.interpreters.ad as ad
-import jax.interpreters.partial_eval as pe
-import jax.interpreters.xla as xla
-import numpy as np
+from jax._src.lib.mlir import ir
+from jax._src.lib.mlir.dialects import hlo
+from jax._src.lib.mlir.dialects import func as func_dialect
+
 
 map, unsafe_map = util.safe_map, map
 zip, unsafe_zip = util.safe_zip, zip
@@ -86,12 +86,12 @@ def shape_tensor(sizes: Sequence[Union[int, ir.RankedTensorType]]
     if type(d) is int:
       return ir_constant(np.array([d], np.int32))
     else:
-      return mhlo.ReshapeOp(int1d, mhlo.ConvertOp(aval_to_ir_type(core.ShapedArray((), np.int32)), d))
+      return hlo.ReshapeOp(int1d, hlo.ConvertOp(aval_to_ir_type(core.ShapedArray((), np.int32)), d))
   d, *ds = map(lower_dim, sizes)
   if not ds:
     return d
   else:
-    return mhlo.ConcatenateOp([d, *ds], i64_attr(0)).result
+    return hlo.ConcatenateOp([d, *ds], i64_attr(0)).result
 
 
 def delegate_lowering(ctx, lowering_fun, *args, **ctx_override_kwargs):
@@ -124,6 +124,12 @@ _dtype_to_ir_type : Dict[np.dtype, Callable[[], ir.Type]] = {
   np.dtype(np.complex128): lambda: ir.ComplexType.get(ir.F64Type.get()),
 }
 
+if xc.mlir_api_version >= 43 and xc._version >= 113:
+  _dtype_to_ir_type.update({
+      np.dtype(dtypes.float8_e4m3fn): ir.Float8E4M3FNType.get,
+      np.dtype(dtypes.float8_e5m2): ir.Float8E5M2Type.get,
+  })
+
 def dtype_to_ir_type(dtype: Union[np.dtype, np.generic]) -> ir.Type:
   assert isinstance(dtype, (np.dtype, np.generic)), type(dtype)
   dtype = np.dtype(dtype)
@@ -143,7 +149,7 @@ def _array_ir_types(aval: Union[core.ShapedArray, core.DShapedArray]
   return (ir.RankedTensorType.get(aval.shape, dtype_to_ir_type(aval.dtype)),)
 
 def _dynamic_array_ir_types(aval: core.ShapedArray) -> Sequence[ir.Type]:
-  dyn_size = ir.ShapedType.get_dynamic_size() if mlir_api_version >= 35 else -1
+  dyn_size = ir.ShapedType.get_dynamic_size()
   shape = [d if type(d) is int else dyn_size for d in aval.shape]
   return (ir.RankedTensorType.get(shape, dtype_to_ir_type(aval.dtype)),)
 
@@ -162,7 +168,7 @@ def aval_to_ir_types(aval: core.AbstractValue) -> Sequence[ir.Type]:
 
 ir_type_handlers[core.ShapedArray] = _array_ir_types
 ir_type_handlers[core.ConcreteArray] = _array_ir_types
-ir_type_handlers[core.AbstractToken] = lambda _: [mhlo.TokenType.get()]
+ir_type_handlers[core.AbstractToken] = lambda _: [hlo.TokenType.get()]
 ir_type_handlers[core.DShapedArray] = _dynamic_array_ir_types
 
 def aval_to_ir_type(aval: core.AbstractValue) -> ir.Type:
@@ -239,9 +245,14 @@ def _numpy_array_constant(x: np.ndarray, canonicalize_types
     x = x.view(np.uint16)
   x = np.ascontiguousarray(x)
   attr = ir.DenseElementsAttr.get(x, type=element_type, shape=shape)
-  return (mhlo.ConstantOp(attr).result,)
+  return (hlo.ConstantOp(attr).result,)
 
 
+def _masked_array_constant_handler(*args, **kwargs):
+  raise ValueError("numpy masked arrays are not supported as direct inputs to JAX functions. "
+                   "Use arr.filled() to convert the value to a standard numpy array.")
+
+register_constant_handler(np.ma.MaskedArray, _masked_array_constant_handler)
 
 def _ndarray_constant_handler(val: np.ndarray, canonicalize_types
                              ) -> Sequence[ir.Value]:
@@ -272,7 +283,7 @@ def _ndarray_constant_handler(val: np.ndarray, canonicalize_types
     if canonicalize_types:
       collapsed_val = np.asarray(
           collapsed_val, dtypes.canonicalize_dtype(collapsed_val.dtype))
-    out = mhlo.BroadcastInDimOp(
+    out = hlo.BroadcastInDimOp(
         ir.RankedTensorType.get(
             val.shape, dtype_to_ir_type(collapsed_val.dtype)),
         _numpy_array_constant(collapsed_val, canonicalize_types=False)[0],
@@ -302,8 +313,12 @@ def _device_array_constant_handler(val, canonicalize_types):
 for t in device_array.device_array_types:
   register_constant_handler(t, _device_array_constant_handler)
 
-register_constant_handler(
-  core.Token, lambda _, __: [mhlo.CreateTokenOp(mhlo.TokenType.get()).result])
+def _token_constant_handler(val, canonicalize_types):
+  if mlir_api_version < 40:
+    return [hlo.CreateTokenOp(hlo.TokenType.get()).result]
+  else:
+    return [hlo.CreateTokenOp().result]
+register_constant_handler(core.Token, _token_constant_handler)
 
 # Source locations
 
@@ -327,12 +342,11 @@ def _source_info_to_location(
 # Translation rules
 def make_ir_context() -> ir.Context:
   """Creates an MLIR context suitable for JAX IR."""
+  from jax._src.lib.mlir import dialects
   context = ir.Context()
-  mhlo.register_mhlo_dialect(context)
-  chlo.register_dialect(context)
-  if mlir_api_version >= 37:
-    from jax._src.lib.mlir.dialects import stablehlo
-    stablehlo.register_dialect(context)
+  dialects.mhlo.register_mhlo_dialect(context)
+  dialects.chlo.register_dialect(context)
+  dialects.stablehlo.register_dialect(context)
   return context
 
 
@@ -574,21 +588,21 @@ class DimPolyEvaluator:
   def __init__(self, value: ir.Value):
     self.value = value
 
-  def __add__(self, other: Union[np.int32, DimPolyEvaluator]):
+  def __add__(self, other: Union[np.int32, np.int64, DimPolyEvaluator]):
     if not isinstance(other, DimPolyEvaluator):
       other = DimPolyEvaluator(ir_constant(other))
-    return DimPolyEvaluator(mhlo.AddOp(self.value, other.value).result)
+    return DimPolyEvaluator(hlo.AddOp(self.value, other.value).result)
 
-  def __radd__(self, other: np.int32):
-    return DimPolyEvaluator(mhlo.AddOp(ir_constant(other), self.value).result)
+  def __radd__(self, other: Union[np.int32, np.int64]):
+    return DimPolyEvaluator(hlo.AddOp(ir_constant(other), self.value).result)
 
-  def __mul__(self, other: Union[np.int32, DimPolyEvaluator]):
+  def __mul__(self, other: Union[np.int32, np.int64, DimPolyEvaluator]):
     if not isinstance(other, DimPolyEvaluator):
       other = DimPolyEvaluator(ir_constant(other))
-    return DimPolyEvaluator(mhlo.MulOp(self.value, other.value).result)
+    return DimPolyEvaluator(hlo.MulOp(self.value, other.value).result)
 
-  def __rmul__(self, other: np.int32):
-    return DimPolyEvaluator(mhlo.MulOp(ir_constant(other), self.value).result)
+  def __rmul__(self, other: Union[np.int32, np.int64]):
+    return DimPolyEvaluator(hlo.MulOp(ir_constant(other), self.value).result)
 
 
 def eval_dynamic_shape(ctx: LoweringRuleContext,
@@ -601,7 +615,7 @@ def eval_dynamic_shape(ctx: LoweringRuleContext,
                    for dv_name, dv_val in zip(ctx.module_context.dim_vars, ctx.dim_var_values)}
     def eval_dim(d: core.DimSize) -> Union[int, ir.Value]:
       try:
-        return int(d)
+        return operator.index(d)
       except:
         if isinstance(d, ir.Value):
           return d
@@ -616,10 +630,7 @@ class LoweringResult(NamedTuple):
   host_callbacks: List[Any]
 
 
-if xla_extension_version >= 102:
-  _platforms_with_donation = ["cpu", "cuda", "rocm", "tpu"]
-else:
-  _platforms_with_donation = ["cuda", "rocm", "tpu"]
+_platforms_with_donation = ["cpu", "cuda", "rocm", "tpu"]
 
 
 def lower_jaxpr_to_module(
@@ -636,7 +647,7 @@ def lower_jaxpr_to_module(
     arg_shardings: Optional[Sequence[Optional[xc.OpSharding]]] = None,
     result_shardings: Optional[Sequence[Optional[xc.OpSharding]]] = None
 ) -> LoweringResult:
-  """Lowers a top-level jaxpr to an MHLO module.
+  """Lowers a top-level jaxpr to an MLIR module.
 
   Handles the quirks of the argument/return value passing conventions of the
   runtime.
@@ -674,7 +685,7 @@ def lower_jaxpr_to_module(
       msg = f"Donation is not implemented for {platform}.\n{msg}"
     warnings.warn(f"Some donated buffers were not usable: {', '.join(unused_donations)}.\n{msg}")
 
-  # MHLO channels need to start at 1
+  # HLO channels need to start at 1
   channel_iter = itertools.count(1)
   # Create a keepalives list that will be mutated during the lowering.
   keepalives: List[Any] = []
@@ -707,8 +718,7 @@ def lower_jaxpr_to_module(
     lower_jaxpr_to_fun(
         ctx, "main", jaxpr, ordered_effects, public=True, create_tokens=True,
         replace_tokens_with_dummy=True,
-        num_output_tokens=(
-          1 if (unordered_effects and not can_execute_with_token) else 0),
+        num_output_tokens=0,
         replicated_args=replicated_args,
         arg_shardings=arg_shardings, result_shardings=result_shardings,
         input_output_aliases=input_output_aliases)
@@ -757,17 +767,20 @@ def _set_up_aliases(avals_in, avals_out, donated_args):
 Token = Sequence[ir.Value]
 
 def token_type() -> Sequence[ir.Type]:
-  return [mhlo.TokenType.get()]
+  return [hlo.TokenType.get()]
 
 def create_token() -> Token:
-  return wrap_singleton_ir_values(
-      mhlo.CreateTokenOp(mhlo.TokenType.get()).result)
+  if mlir_api_version < 40:
+    return wrap_singleton_ir_values(
+        hlo.CreateTokenOp(hlo.TokenType.get()).result)
+  else:
+    return wrap_singleton_ir_values(hlo.CreateTokenOp().result)
 
 class TokenSet:
   """An immutable container of tokens to be used to lower effectful jaxprs. When lowering
-  effectful jaxprs, we need to thread MHLO tokens to sequence them. Each effect
+  effectful jaxprs, we need to thread HLO tokens to sequence them. Each effect
   will need its own token that will be threaded in and out of the effectful
-  primitives. A `TokenSet` encapsulates a set of MHLO tokens that will be
+  primitives. A `TokenSet` encapsulates a set of HLO tokens that will be
   used by the lowering rules.
   """
   _tokens: typing.OrderedDict[core.Effect, Token]
@@ -831,6 +844,7 @@ def lower_jaxpr_to_fun(
     use_sharding_annotations: bool = True,
     input_output_aliases: Optional[Sequence[Optional[int]]] = None,
     num_output_tokens: int = 0,
+    api_name: str = 'jit',
 ) -> func_dialect.FuncOp:
   """Lowers jaxpr and its callees to an IR function.
 
@@ -843,20 +857,22 @@ def lower_jaxpr_to_fun(
     jaxpr: the jaxpr to lower.
     effects: a sequence of `core.Effect`s corresponding to an ordering of tokens
       that will be created in or used by the lowered function.
-    create_tokens: if true, the MHLO will create tokens and ignore dummy input tokens.
+    create_tokens: if true, the HLO will create tokens and ignore dummy input tokens.
     public: if true, the function's visibility is set to "public".
     replace_tokens_with_dummy: if true, token arguments/return values are
       replaced with bool arrays of size [0].
     replicated_args: if present, annotates arguments as replicated.
     arg_shardings: sharding annotations for each argument (optional).
     result_shardings: sharding annotations for each argument (optional).
-    use_sharding_annotations: if True, use mhlo.sharding annotations on
+    use_sharding_annotations: if True, use "mhlo.sharding" annotations on
       parameters and return values to express sharding. If False, use
-      mhlo.custom_call operators with sharding annotations.
-      TODO(b/228598865): remove this option when mhlo.sharding annotations are
-      propagated on non-entry functions during MHLO->HLO conversion.
+      hlo.custom_call operators with sharding annotations.
+      TODO(b/228598865): remove this option when "mhlo.sharding" annotations are
+      propagated on non-entry functions during MLIR->HLO conversion.
     input_output_aliases: optional sequence that maps argument numbers to the
       corresponding output that should alias them.
+    api_name: The name of the higher level primitive which should show up in the
+      name stack.
   Returns the name of the function.
   """
   def aval_to_types(aval):
@@ -865,7 +881,7 @@ def lower_jaxpr_to_fun(
     return aval_to_ir_types(aval)
 
   num_dim_vars = len(ctx.dim_vars)
-  dim_var_types = map(aval_to_types, [core.ShapedArray((), np.int32)] * num_dim_vars)
+  dim_var_types = map(aval_to_types, [core.ShapedArray((), dtypes.canonicalize_dtype(np.int64))] * num_dim_vars)
 
   # Function inputs: *dim_var_values, *tokens, *actual_inputs
   input_types = map(aval_to_types, jaxpr.in_avals)
@@ -980,11 +996,14 @@ def lower_jaxpr_to_fun(
     args: List[List[ir.Value]] = []
     for aval, arg in zip(jaxpr.in_avals, unflattened_args):
       if replace_tokens_with_dummy and aval is core.abstract_token:
-        args.append(mhlo.CreateTokenOp(mhlo.TokenType.get()).results)
+        if mlir_api_version < 40:
+          args.append(hlo.CreateTokenOp(hlo.TokenType.get()).results)
+        else:
+          args.append(hlo.CreateTokenOp().results)
       else:
         args.append(arg)
     callee_name_stack = xla.extend_name_stack(ctx.name_stack,
-                                              util.wrap_name(name, 'jit'))
+                                              util.wrap_name(name, api_name))
     out_vals, tokens_out = jaxpr_subcomp(ctx.replace(name_stack=callee_name_stack),
                                          jaxpr.jaxpr, tokens_in, map(ir_constants, jaxpr.consts),
                                          *args, dim_var_values=dim_var_values)
@@ -1016,7 +1035,7 @@ def _emit_lowering_rule_as_fun(lowering_rule,
   """Emits the contents of a lowering rule as a private function."""
   num_dim_vars = len(ctx.module_context.dim_vars)
   # TODO(necula) maybe only pass the dim_vars if they are needed?
-  dim_var_types = map(aval_to_ir_types, [core.ShapedArray((), np.int32)] * num_dim_vars)
+  dim_var_types = map(aval_to_ir_types, [core.ShapedArray((), dtypes.canonicalize_dtype(np.int64))] * num_dim_vars)
 
   input_types = map(aval_to_ir_types, ctx.avals_in)
   output_types = map(aval_to_ir_types, ctx.avals_out)
@@ -1051,7 +1070,7 @@ def jaxpr_subcomp(ctx: ModuleContext, jaxpr: core.Jaxpr,
                   *args: Sequence[ir.Value],
                   dim_var_values: Sequence[ir.Value]
                   ) -> Tuple[Sequence[Sequence[ir.Value]], TokenSet]:
-  """Lowers a jaxpr into mHLO, inlined into an existing function.
+  """Lowers a jaxpr into MLIR, inlined into an existing function.
 
   Assumes that an MLIR context, location, and insertion point are set.
 
@@ -1259,13 +1278,13 @@ def broadcast_in_dim(ctx: LoweringRuleContext, op, aval_out: core.AbstractValue,
         broadcast_dimensions=broadcast_dimensions)
   if not core.is_constant_shape(aval_out.shape):  # type: ignore
     shape = eval_dynamic_shape(ctx, aval_out.shape)  # type: ignore
-    return mhlo.DynamicBroadcastInDimOp(
+    return hlo.DynamicBroadcastInDimOp(
         aval_to_ir_type(aval_out), op,
         shape_tensor(shape),
         dense_int_elements(broadcast_dimensions),
     ).result
   else:
-    return mhlo.BroadcastInDimOp(
+    return hlo.BroadcastInDimOp(
         aval_to_ir_type(aval_out), op,
         dense_int_elements(broadcast_dimensions)).result
 
@@ -1292,12 +1311,12 @@ def reshape(ctx: LoweringRuleContext, op, aval_out: core.AbstractValue) -> ir.Va
     aval_out, = aval_out.dtype._rules.physical_avals(aval_out)  # type: ignore
   if not core.is_constant_shape(aval_out.shape):  # type: ignore
     shape = eval_dynamic_shape(ctx, aval_out.shape)  # type: ignore
-    return mhlo.DynamicReshapeOp(
+    return hlo.DynamicReshapeOp(
         aval_to_ir_type(aval_out), op,
         shape_tensor(shape),
     ).result
   else:
-    return mhlo.ReshapeOp(aval_to_ir_type(aval_out), op).result
+    return hlo.ReshapeOp(aval_to_ir_type(aval_out), op).result
 
 def slice_op(ctx: LoweringRuleContext, x, aval_out, *,
              start_indices, limit_indices, strides) -> ir.Value:
@@ -1309,16 +1328,16 @@ def slice_op(ctx: LoweringRuleContext, x, aval_out, *,
     start_indices = eval_dynamic_shape(ctx, start_indices)
     limit_indices = eval_dynamic_shape(ctx, limit_indices)
     strides = eval_dynamic_shape(ctx, strides)
-    return mhlo.RealDynamicSliceOp(aval_to_ir_type(aval_out),
-                                   x,
-                                   shape_tensor(start_indices),
-                                   shape_tensor(limit_indices),
-                                   shape_tensor(strides)).result
+    return hlo.RealDynamicSliceOp(aval_to_ir_type(aval_out),
+                                  x,
+                                  shape_tensor(start_indices),
+                                  shape_tensor(limit_indices),
+                                  shape_tensor(strides)).result
   else:
-    return mhlo.SliceOp(x,
-                        dense_int_elements(start_indices),
-                        dense_int_elements(limit_indices),
-                        dense_int_elements(strides)).result
+    return hlo.SliceOp(x,
+                       dense_int_elements(start_indices),
+                       dense_int_elements(limit_indices),
+                       dense_int_elements(strides)).result
 
 def dynamic_slice(ctx: LoweringRuleContext, aval_out, x, *,
                   start_indices) -> ir.Value:
@@ -1328,15 +1347,16 @@ def dynamic_slice(ctx: LoweringRuleContext, aval_out, x, *,
   slice_sizes = aval_out.shape
   if not core.is_constant_shape(slice_sizes):
     slice_sizes = eval_dynamic_shape(ctx, slice_sizes)
-    return mhlo.RealDynamicSliceOp(
+    return hlo.RealDynamicSliceOp(
         aval_to_ir_type(aval_out), x,
         shape_tensor(start_indices),
-        shape_tensor(slice_sizes),
+        hlo.AddOp(shape_tensor(start_indices),
+                  shape_tensor(slice_sizes)).result,
         shape_tensor([1] * len(slice_sizes))
     ).result
   else:
-    return mhlo.DynamicSliceOp(x, start_indices,
-                               dense_int_elements(slice_sizes)).result
+    return hlo.DynamicSliceOp(x, start_indices,
+                              dense_int_elements(slice_sizes)).result
 
 def dynamic_update_slice(ctx: LoweringRuleContext, aval_out, x, update, *,
                          start_indices) -> ir.Value:
@@ -1345,8 +1365,28 @@ def dynamic_update_slice(ctx: LoweringRuleContext, aval_out, x, update, *,
         ctx, aval_out, x, update, *start_indices)
 
   # TODO(necula): handle dynamic shapes
-  return mhlo.DynamicUpdateSliceOp(aval_to_ir_type(aval_out), x, update,
-                                  start_indices).result
+  if mlir_api_version < 40:
+    return hlo.DynamicUpdateSliceOp(
+        aval_to_ir_type(aval_out), x, update, start_indices).result
+  else:
+    return hlo.DynamicUpdateSliceOp(x, update, start_indices).result
+
+def pad(ctx: LoweringRuleContext, aval_out,
+        x, padding_value,
+        padding_low, padding_high, padding_interior) -> ir.Value:
+  if all(core.is_constant_shape(s) for s in (padding_low,
+                                             padding_high, padding_interior)):
+    return hlo.PadOp(x, padding_value,
+                     dense_int_elements(padding_low),
+                     dense_int_elements(padding_high),
+                     dense_int_elements(padding_interior)).result
+  else:
+    padding_low = shape_tensor(eval_dynamic_shape(ctx, padding_low))
+    padding_high = shape_tensor(eval_dynamic_shape(ctx, padding_high))
+    padding_interior = shape_tensor(eval_dynamic_shape(ctx, padding_interior))
+    return hlo.DynamicPadOp(
+        aval_to_ir_type(aval_out),
+        x, padding_value, padding_low, padding_high, padding_interior).result
 
 def full_like_aval(ctx: LoweringRuleContext, value, aval: core.ShapedArray) -> ir.Value:
   """Returns an IR constant shaped full of `value` shaped like `aval`."""
@@ -1360,14 +1400,14 @@ def zeros_like_lowering(ctx, x):
 register_lowering(ad_util.zeros_like_p, zeros_like_lowering)
 
 def add_jaxvals_lowering(ctx, x, y):
-  return mhlo.AddOp(x, y).results
+  return hlo.AddOp(x, y).results
 register_lowering(ad_util.add_jaxvals_p, add_jaxvals_lowering)
 
 register_lowering(ad_util.stop_gradient_p, lambda ctx, x: [x])
 
 
-def compare_mhlo(x, y, direction: str, comparison_type: Optional[str] = None):
-  """Creates mhlo.CompareOp."""
+def compare_hlo(x, y, direction: str, comparison_type: Optional[str] = None):
+  """Creates CompareOp."""
   if comparison_type is None:
     elem_type = ir.RankedTensorType(x.type).element_type
     if ir.IntegerType.isinstance(elem_type):
@@ -1376,34 +1416,34 @@ def compare_mhlo(x, y, direction: str, comparison_type: Optional[str] = None):
     else:
       comparison_type = "FLOAT"
 
-  return mhlo.CompareOp(
+  return hlo.CompareOp(
       x,
       y,
-      mhlo.ComparisonDirectionAttr.get(direction),
-      compare_type=mhlo.ComparisonTypeAttr.get(comparison_type))
+      hlo.ComparisonDirectionAttr.get(direction),
+      compare_type=hlo.ComparisonTypeAttr.get(comparison_type))
 
-def _minmax_mhlo(op, cmp, x, y):
+def _minmax_hlo(op, cmp, x, y):
   """Min/max that compares complex values lexicographically as pairs."""
   tensor_type = ir.RankedTensorType(x.type)
   if ir.ComplexType.isinstance(tensor_type.element_type):
-    rx = mhlo.RealOp(x).result
-    ry = mhlo.RealOp(y).result
-    real_eq = compare_mhlo(rx, ry, "EQ", "FLOAT")
-    real_cmp = compare_mhlo(rx, ry, cmp, "FLOAT")
-    imag_cmp = compare_mhlo(
-        mhlo.ImagOp(x).result,
-        mhlo.ImagOp(y).result, cmp, "FLOAT")
-    which = mhlo.SelectOp(real_eq, imag_cmp, real_cmp).result
-    return mhlo.SelectOp(which, x, y)
+    rx = hlo.RealOp(x).result
+    ry = hlo.RealOp(y).result
+    real_eq = compare_hlo(rx, ry, "EQ", "FLOAT")
+    real_cmp = compare_hlo(rx, ry, cmp, "FLOAT")
+    imag_cmp = compare_hlo(
+        hlo.ImagOp(x).result,
+        hlo.ImagOp(y).result, cmp, "FLOAT")
+    which = hlo.SelectOp(real_eq, imag_cmp, real_cmp).result
+    return hlo.SelectOp(which, x, y)
   else:
     return op(x, y)
 
-min_mhlo = partial(_minmax_mhlo, mhlo.MinOp, "LT")
-max_mhlo = partial(_minmax_mhlo, mhlo.MaxOp, "GT")
+min_hlo = partial(_minmax_hlo, hlo.MinOp, "LT")
+max_hlo = partial(_minmax_hlo, hlo.MaxOp, "GT")
 
 
-def convert_mhlo(ctx: LoweringRuleContext, x, aval_in, aval_out):
-  """Variant of convert that has XLA HLO semantics.
+def convert_hlo(ctx: LoweringRuleContext, x, aval_in, aval_out):
+  """Variant of convert that has HLO semantics.
 
   In particular, treat casts to boolean as x != 0, rather than truncating
   integer values (b/209440332)."""
@@ -1415,9 +1455,9 @@ def convert_mhlo(ctx: LoweringRuleContext, x, aval_in, aval_out):
       compare_type = "SIGNED"
     else:
       compare_type = "UNSIGNED"
-    return compare_mhlo(x, full_like_aval(ctx, 0, aval_in), "NE",
-                         compare_type).result
-  return mhlo.ConvertOp(aval_to_ir_type(aval_out), x).result
+    return compare_hlo(x, full_like_aval(ctx, 0, aval_in), "NE",
+                       compare_type).result
+  return hlo.ConvertOp(aval_to_ir_type(aval_out), x).result
 
 def _wrap_with_spmd_op(name: str,
                        result_type: ir.Type,
@@ -1431,14 +1471,14 @@ def _wrap_with_spmd_op(name: str,
         [str(i) for i in sorted(unspecified_dims)]) + "]"
   else:
     backend_config = ""
-  op = mhlo.CustomCallOp([result_type], [x],
-                         call_target_name=ir.StringAttr.get(name),
-                         has_side_effect=ir.BoolAttr.get(False),
-                         backend_config=ir.StringAttr.get(backend_config),
-                         api_version=i32_attr(1),
-                         called_computations=ir.ArrayAttr.get([]),
-                         operand_layouts=None,
-                         result_layouts=None)
+  op = hlo.CustomCallOp([result_type], [x],
+                        call_target_name=ir.StringAttr.get(name),
+                        has_side_effect=ir.BoolAttr.get(False),
+                        backend_config=ir.StringAttr.get(backend_config),
+                        api_version=i32_attr(1),
+                        called_computations=ir.ArrayAttr.get([]),
+                        operand_layouts=None,
+                        result_layouts=None)
   op.attributes["mhlo.sharding"] = ir.StringAttr.get(
       sharding_proto.SerializeToString())
   return op.result
@@ -1476,7 +1516,7 @@ def cache_lowering(f):
     except TypeError:
       # If the parameters aren't hashable, give up on caching.
       # TODO(phawkins): switch to requiring hashability, when XLA fallback
-      # computations have been ported to MHLO.
+      # computations have been ported to MLIR.
       return f(ctx, *args, **params)
     if func is None:
       func = _emit_lowering_rule_as_fun(partial(f, **params), ctx)
@@ -1493,12 +1533,12 @@ def cache_lowering(f):
 
 
 
-def xla_computation_to_mhlo_module(xla_computation: xc.XlaComputation
+def xla_computation_to_mlir_module(xla_computation: xc.XlaComputation
                                   ) -> ir.Module:
   module_str = xc._xla.mlir.xla_computation_to_mlir_module(xla_computation)
   return ir.Module.parse(module_str)
 
-def merge_mhlo_modules(dst_module: ir.Module,
+def merge_mlir_modules(dst_module: ir.Module,
                        sym_name: str,
                        src_module: ir.Module) -> str:
   """Returns the name of src_module's main() function, after renaming."""
@@ -1541,11 +1581,17 @@ def xla_fallback_lowering(prim: core.Primitive):
       axis_env = axis_ctx.unsafe_axis_env
     else:
       axis_env = module_ctx.axis_env
+
+    if any(hasattr(a, "shape") and
+           not core.is_constant_shape(a.shape) for a in (ctx.avals_in + ctx.avals_out)):
+      raise NotImplementedError(
+          f"Shape polymorphism for xla_fallback_lowering is not implemented ({ctx.primitive}); b/261682623")
+
     xla_computation = xla.primitive_subcomputation(
         module_ctx.platform, axis_env, prim, ctx.avals_in,
         ctx.avals_out, **params)
-    xla_module = xla_computation_to_mhlo_module(xla_computation)
-    callee_name = merge_mhlo_modules(
+    xla_module = xla_computation_to_mlir_module(xla_computation)
+    callee_name = merge_mlir_modules(
         module_ctx.module, f"xla_fallback_{prim.name}", xla_module)
     output_types = map(aval_to_ir_types, ctx.avals_out)
     flat_output_types = util.flatten(output_types)
@@ -1557,7 +1603,7 @@ def xla_fallback_lowering(prim: core.Primitive):
                                flatten_lowering_ir_args(args)).result
     if not prim.multiple_results:
       return [call]
-    flat_results = [mhlo.GetTupleElementOp(call, i32_attr(i)).result
+    flat_results = [hlo.GetTupleElementOp(call, i32_attr(i)).result
                     for i in range(len(flat_output_types))]
 
     return util.unflatten(flat_results, map(len, output_types))
@@ -1592,12 +1638,16 @@ def _dtype_to_xla_type_string(dtype: np.dtype) -> str:
     raise NotImplementedError(dtype)
   return _dtype_to_xla_type_string_map[dtype]
 
-def send_to_host(channel: int, token: mhlo.TokenType, operand: Any,
+def send_to_host(channel: int, token: hlo.TokenType, operand: Any,
                  aval: core.ShapedArray, name: str, *,
                  sharding: Optional[xc.OpSharding] = None) -> ir.Value:
-  channel_handle = mhlo.ChannelHandle.get(channel, SEND_TO_HOST_TYPE)
-  send_op = mhlo.SendOp(mhlo.TokenType.get(), [operand], token, channel_handle,
-                        is_host_transfer=ir.BoolAttr.get(True))
+  channel_handle = hlo.ChannelHandle.get(channel, SEND_TO_HOST_TYPE)
+  if mlir_api_version < 40:
+    send_op = hlo.SendOp(hlo.TokenType.get(), [operand], token, channel_handle,
+                         is_host_transfer=ir.BoolAttr.get(True))
+  else:
+    send_op = hlo.SendOp([operand], token, channel_handle,
+                         is_host_transfer=ir.BoolAttr.get(True))
   dtype_str = _dtype_to_xla_type_string(aval.dtype)
   if dtype_str in {"f64", "s64", "u64", "c64", "c128"}:
     raise NotImplementedError("64-bit types not supported.")
@@ -1611,13 +1661,13 @@ def send_to_host(channel: int, token: mhlo.TokenType, operand: Any,
   return send_op.result
 
 
-def receive_from_host(channel: int, token: mhlo.TokenType,
+def receive_from_host(channel: int, token: hlo.TokenType,
                       out_aval: core.ShapedArray, name: str, *,
                       sharding: Optional[xc.OpSharding] = None) -> ir.Value:
-  channel_handle = mhlo.ChannelHandle.get(channel, RECV_FROM_HOST_TYPE)
-  recv_op = mhlo.RecvOp([aval_to_ir_type(out_aval),
-                         mhlo.TokenType.get()], token, channel_handle,
-                         is_host_transfer=ir.BoolAttr.get(True))
+  channel_handle = hlo.ChannelHandle.get(channel, RECV_FROM_HOST_TYPE)
+  recv_op = hlo.RecvOp([aval_to_ir_type(out_aval),
+                        hlo.TokenType.get()], token, channel_handle,
+                        is_host_transfer=ir.BoolAttr.get(True))
   dtype_str = _dtype_to_xla_type_string(out_aval.dtype)
   if dtype_str in {"f64", "s64", "u64", "c64", "c128"}:
     raise NotImplementedError("64-bit types not supported.")
@@ -1646,7 +1696,10 @@ def _emit_tpu_python_callback(
     *,
     sharding: Optional[xc.OpSharding] = None
 ) -> Tuple[List[ir.Value], Any, Any]:
-  token = token or mhlo.CreateTokenOp(mhlo.TokenType.get()).result
+  if mlir_api_version < 40:
+    token = token or hlo.CreateTokenOp(hlo.TokenType.get()).result
+  else:
+    token = token or hlo.CreateTokenOp().result
   _wrapped_callback = callback
 
   send_channels = []
@@ -1654,7 +1707,7 @@ def _emit_tpu_python_callback(
     # If there are no operands to the callback, we need to insert a dummy send
     # op or the callback will never be triggered!
     # TODO(sharadmv,chky): Enable this fix in the runtime as opposed to in
-    # MHLO builder.
+    # MLIR builder.
     callback_without_args = _wrapped_callback
     def _wrapped_callback(*args):  # pylint: disable=function-redefined
       del args
@@ -1679,36 +1732,16 @@ def _emit_tpu_python_callback(
 
   recv_channels = []
   outputs = []
-  # `send-to-host`s can be interleaved by the transfer manager so we add in a
-  # dummy recv to sequence them (the recv can only happen after all the sends
-  # are done). We'd like to send back a 0-shaped array to avoid unnecessary
-  # copies but that currently doesn't work with the transfer
-  # manager as well.
-  # TODO(b/238239458): enable sending back a 0-dim array
-  # TODO(b/238239928): avoid interleaving sends in the transfer manager
-  if not result_avals:
-    callback_without_return_values = _wrapped_callback
-    def _wrapped_callback(*args):  # pylint: disable=function-redefined
-      callback_without_return_values(*args)
-      return (np.zeros(1, np.float32),)
-    recv_channel = ctx.module_context.new_channel()
-    dummy_recv_aval = core.ShapedArray((1,), np.float32)
-    result_shapes = [*result_shapes,
-                     xla.aval_to_xla_shapes(dummy_recv_aval)[0]]
-    token, _ = receive_from_host(recv_channel, token, dummy_recv_aval,
-                                 callback.__name__, sharding=sharding)
-    recv_channels.append(recv_channel)
-  else:
-    for result_aval in result_avals:
-      if any(s == 0 for s in result_aval.shape):
-        raise NotImplementedError(
-            "Callbacks with zero-dimensional values not supported on TPU.")
-      channel = ctx.module_context.new_channel()
-      assert isinstance(result_aval, core.ShapedArray)
-      token, out = receive_from_host(channel, token, result_aval,
-                                     callback.__name__, sharding=sharding)
-      outputs.append(out)
-      recv_channels.append(channel)
+  for result_aval in result_avals:
+    if any(s == 0 for s in result_aval.shape):
+      raise NotImplementedError(
+          "Callbacks with zero-dimensional values not supported on TPU.")
+    channel = ctx.module_context.new_channel()
+    assert isinstance(result_aval, core.ShapedArray)
+    token, out = receive_from_host(channel, token, result_aval,
+                                   callback.__name__, sharding=sharding)
+    outputs.append(out)
+    recv_channels.append(channel)
   opaque = backend.make_python_callback_from_host_send_and_recv(
       _wrapped_callback, operand_shapes, result_shapes, send_channels,
       recv_channels)
@@ -1735,7 +1768,7 @@ def emit_python_callback(
     operand_layouts: Optional[Sequence[Optional[Sequence[int]]]] = None,
     result_layouts: Optional[Sequence[Optional[Sequence[int]]]] = None,
     ) -> Tuple[List[ir.Value], Any, Any]:
-  """Emits MHLO that calls back to a provided Python function."""
+  """Emits MLIR that calls back to a provided Python function."""
   platform = ctx.module_context.platform
   if platform not in {"cpu", "cuda", "rocm", "tpu"}:
     raise ValueError(
@@ -1797,13 +1830,8 @@ def emit_python_callback(
     ]
     operands = [token, *operands]
     result_types = [token_type()[0], *result_types]
-    if xla_extension_version >= 105:
-      operand_mlir_layouts = [_layout_to_mlir_layout(None), *operand_mlir_layouts]
-      result_mlir_layouts = [_layout_to_mlir_layout(None), *result_mlir_layouts]
-    else:
-      # Token layouts aren't converted correctly into HLO in older XLA versions.
-      operand_mlir_layouts = None  # type: ignore
-      result_mlir_layouts = None  # type: ignore
+    operand_mlir_layouts = [_layout_to_mlir_layout(None), *operand_mlir_layouts]
+    result_mlir_layouts = [_layout_to_mlir_layout(None), *result_mlir_layouts]
   callback_descriptor, keepalive = (
       backend.get_emit_python_callback_descriptor(_wrapped_callback,
                                                   operand_shapes,
@@ -1816,7 +1844,7 @@ def emit_python_callback(
   result_type = ir.TupleType.get_tuple(result_types)
   call_target_name = ("xla_python_gpu_callback"
                      if platform in {"cuda", "rocm"} else "xla_python_cpu_callback")
-  result = mhlo.CustomCallOp(
+  result = hlo.CustomCallOp(
       [result_type],
       callback_operands,
       call_target_name=ir.StringAttr.get(call_target_name),
@@ -1833,7 +1861,7 @@ def emit_python_callback(
   if sharding is not None:
     set_sharding(result, sharding)
   results = [
-      mhlo.GetTupleElementOp(result, i32_attr(i)).result
+      hlo.GetTupleElementOp(result, i32_attr(i)).result
       for i in range(len(result_types))
   ]
   if token:
