@@ -12,6 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# Note: import <name> as <name> is required for names to be exported.
+# See PEP 484 & https://github.com/google/jax/issues/7570
+
 from jax.scipy.stats import bernoulli as bernoulli
 from jax.scipy.stats import beta as beta
 from jax.scipy.stats import cauchy as cauchy
@@ -35,3 +38,4 @@ from jax.scipy.stats import gennorm as gennorm
 from jax.scipy.stats import truncnorm as truncnorm
 from jax._src.scipy.stats.kde import gaussian_kde as gaussian_kde
 from jax._src.scipy.stats._core import mode as mode
+from jax.scipy.stats import vonmises as vonmises
